package com.gic.haoban.manage.service.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Set;

import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.service.service.ExternalClerkRelatedService;
import com.gic.haoban.manage.service.service.StaffClerkRelationService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.haoban.manage.service.dao.mapper.TabHaobanClerkMainStoreRelatedMapper;
import com.gic.haoban.manage.service.entity.TabHaobanClerkMainStoreRelated;
import com.gic.haoban.manage.service.service.ClerkMainStoreRelatedService;

@Service
public class ClerkMainStoreRelatedServiceImpl implements ClerkMainStoreRelatedService {
	
	@Autowired
	private TabHaobanClerkMainStoreRelatedMapper mapper;

	@Autowired
	private StaffClerkRelationService staffClerkRelationService;

	@Autowired
	private ExternalClerkRelatedService externalClerkRelatedService;

	@Override
	public void setMainStore(String staffId, String storeId, String wxEnterpriseId) {
		String uuId = com.gic.haoban.common.utils.StringUtil.randomUUID();
		Date now = new Date();
		TabHaobanClerkMainStoreRelated t = new TabHaobanClerkMainStoreRelated();
		t.setStaffId(staffId);
		t.setWxEnterpriseId(wxEnterpriseId);
		t.setStoreId(storeId);
		t.setCreateTime(now);
		t.setUpdateTime(now);
		t.setStatusFlag(1);
        t.setFreshFriendFlag(0);
        t.setClerkMainStoreRelatedId(uuId);
        mapper.insert(t);
		
	}

	@Override
	public TabHaobanClerkMainStoreRelated selectByWxEnterpriseIdAndStoreId(String staffId, String wxEnterpriseId) {
		return mapper.selectByWxEnterpriseIdAndStoreId(staffId,wxEnterpriseId);
	}

	@Override
	public TabHaobanClerkMainStoreRelated getMainStoreByStaffId(String staffId, String wxEnterpriseId) {
		TabHaobanClerkMainStoreRelated mainStoreRelated = mapper.selectByWxEnterpriseIdAndStoreId(staffId, wxEnterpriseId);
		if (null != mainStoreRelated) {
			return mainStoreRelated;
		}
		return setStaffMainStore(staffId, wxEnterpriseId);
	}

	/**
	 * 随机设置主门店
	 *
	 * @param staffId
	 * @param wxEnterpriseId
	 * @return
	 */
	private TabHaobanClerkMainStoreRelated setStaffMainStore(String staffId, String wxEnterpriseId) {
		List<StaffClerkRelationDTO> clerkRelationDTOS = staffClerkRelationService.lisByStaffId(wxEnterpriseId, staffId);
		if (CollectionUtils.isEmpty(clerkRelationDTOS)) {
			return null;
		}
		StaffClerkRelationDTO staffClerkRelationDTO = clerkRelationDTOS.get(0);
		String lockKey = "set-main-store:" + staffId;
		RedisUtil.lock(lockKey, 3L);
		TabHaobanClerkMainStoreRelated mainStoreRelated = mapper.selectByWxEnterpriseIdAndStoreId(staffId, wxEnterpriseId);
		if (null == mainStoreRelated) {
			this.setMainStore(staffId, staffClerkRelationDTO.getStoreId(), wxEnterpriseId);
			mainStoreRelated = mapper.selectByWxEnterpriseIdAndStoreId(staffId, wxEnterpriseId);
		}
		RedisUtil.unlock(lockKey);
		return mainStoreRelated;
	}

	@Override
	public void delMainStore(String staffId, String storeId, String wxEnterpriseId) {
		//删除主门店
		mapper.delMainStore(staffId, storeId, wxEnterpriseId);
		//删除主门店外部联系人 好友
        externalClerkRelatedService.delByStoreIdAndStaffId(storeId, staffId);
        //设置主门店
		this.setStaffMainStore(staffId, wxEnterpriseId);
	}

	@Override
	public void delMainStoreByStoreIds(Set<String> storeIds) {
		//删除主门店
		mapper.delByStoreIds(storeIds);
		//删除外部联系人
		externalClerkRelatedService.delByStoreIds(storeIds);
	}

	@Override
	public void update(TabHaobanClerkMainStoreRelated r) {
		r.setUpdateTime(new Date());
		mapper.updateByPrimaryKeySelective(r);
	}

	@Override
	public List<TabHaobanClerkMainStoreRelated> listByWxEnterpriseId(String wxEnterpriseId) {
		return mapper.listByWxEnterpriseId(wxEnterpriseId);
	}

}
