package com.gic.haoban.manage.service.service.out.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.haoban.manage.api.dto.CheckFriendSettingDTO;
import com.gic.haoban.manage.api.dto.ExternalUserDTO;
import com.gic.haoban.manage.api.dto.MemberUnionidRelatedDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.CheckQywxSettingApiService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/5/31.
 *
 * @author hua
 */
@Service
public class CheckQywxSettingApiServiceImpl implements CheckQywxSettingApiService {

    private static final Logger logger = getLogger(CheckQywxSettingApiServiceImpl.class);
    private static final String FRIEND_SETTING_KEY = "haoban-check-friend-setting";
    private static final String FRIEND_SETTING_VAL_KEY = "haoban-check-friend-setting-val";

    private static final String SELF_APP = "selfSuiteId";
    private static final String THIRD_APP = "thirdSuiteId";

    private static final String ALERT_KEY = "friend-alert:";

    private static final String ALERT_URL = "https://oapi.dingtalk.com/robot/send?access_token=435c0ddaacda849e145b64cc3a83c37f742105d88cf27b719b2dbef167436ad4";

    @Autowired
    private WxEnterpriseService wxEnterpriseService;

    @Override
    public void setFriendSetting(String wxEnterpriseId, String name) {
        CheckFriendSettingDTO checkFriendSettingDTO = new CheckFriendSettingDTO();
        checkFriendSettingDTO.setName(name);
        checkFriendSettingDTO.setWxEnterpriseId(wxEnterpriseId);
        RedisUtil.setCache(FRIEND_SETTING_KEY, checkFriendSettingDTO);
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<Object, Object> map = redisClient.getMap(FRIEND_SETTING_VAL_KEY);
        map.delete();
    }

    @Override
    public CheckFriendSettingDTO getFriendSetting() {
        return RedisUtil.getCache(FRIEND_SETTING_KEY, CheckFriendSettingDTO.class);
    }

    @Override
    public void saveFriendCallback(String wxEnterpriseId, ExternalUserDTO externalUserDTO, MemberUnionidRelatedDTO dto) {
        if (null == externalUserDTO) {
            messageAlert(wxEnterpriseId, dto, null);
            //todo 告警
            return;
        }
        CheckFriendSettingDTO friendSetting = this.getFriendSetting();
        if (!friendSetting.getName().equals(externalUserDTO.getName())) {
            return;
        }
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<Object, Object> map = redisClient.getMap(FRIEND_SETTING_VAL_KEY);
        if (SELF_APP.equals(dto.getSuiteid())) {
            map.put(SELF_APP, externalUserDTO);
        } else {
            map.put(THIRD_APP, externalUserDTO);
        }
        messageAlert(wxEnterpriseId, dto, "好友提醒：" + dto.getSuiteid());
    }

    /**
     * 告警
     *
     * @param wxEnterpriseId
     * @param dto
     */
    private void messageAlert(String wxEnterpriseId, MemberUnionidRelatedDTO dto, String titleSub) {
        titleSub = StringUtils.isBlank(titleSub) ? "加好友异常" : titleSub;
        String key = ALERT_KEY + wxEnterpriseId;
        RBucket<Object> bucket = RedisUtil.getRedisClient().getBucket(key);
        boolean b = bucket.trySet(1, 60L, TimeUnit.SECONDS);
        if (!b) {
            return;
        }
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        ProviderLocalTag providerLocalTag = ProviderLocalTag.tag.get();
        String traceId = providerLocalTag.traceId;
        String traceUrl = "http://212.64.4.231:8780/damo-stats/mongodb/getLogMessagesView?trace_id=" + traceId;

        String title = wxEnterpriseDTO.getCorpName() + ": " + titleSub;
        Map<String, Object> map = new HashMap<>();
        Map<String, Object> link = new HashMap<>();
        map.put("msgtype", "link");
        link.put("title", title);
        link.put("text", JSONObject.toJSONString(dto));
        link.put("messageUrl", traceUrl);
        map.put("link", link);
        String post = HttpUtil.post(ALERT_URL, JSONObject.toJSONString(map));
        logger.info("alert:{}", post);
    }
}
